use super::*;
use crate::ir::tree::Snapshot;
use crate::ir::value::Value;

#[test]
fn selection1_latest() {
    let query = r#"SELECT "product_code" FROM "hash_testing"
        WHERE "identification_number" in
        (SELECT "identification_number" FROM "hash_testing_hist" WHERE "product_code" = 'b') and "product_code" < 'a'"#;

    let expected = PatternWithParams::new(
        format!(
            "{} {} {} {} {}",
            r#"SELECT "hash_testing"."product_code" FROM "hash_testing""#,
            r#"WHERE ("hash_testing"."product_code") < (?)"#,
            r#"and ("hash_testing"."identification_number") in"#,
            r#"(SELECT "hash_testing_hist"."identification_number" FROM "hash_testing_hist""#,
            r#"WHERE ("hash_testing_hist"."product_code") = (?))"#,
        ),
        vec![Value::from("a"), Value::from("b")],
    );
    check_sql_with_snapshot(query, expected, Snapshot::Latest);
}

#[test]
fn selection1_oldest() {
    let query = r#"SELECT "product_code" FROM "hash_testing"
        WHERE "identification_number" in
        (SELECT "identification_number" FROM "hash_testing_hist" WHERE "product_code" = 'b') and "product_code" < 'a'"#;

    let expected = PatternWithParams::new(
        format!(
            "{} {} {} {} {}",
            r#"SELECT "hash_testing"."product_code" FROM "hash_testing""#,
            r#"WHERE ("hash_testing"."identification_number") in"#,
            r#"(SELECT "hash_testing_hist"."identification_number" FROM "hash_testing_hist""#,
            r#"WHERE ("hash_testing_hist"."product_code") = (?))"#,
            r#"and ("hash_testing"."product_code") < (?)"#,
        ),
        vec![Value::from("b"), Value::from("a")],
    );
    check_sql_with_snapshot(query, expected, Snapshot::Oldest);
}

#[test]
fn selection2_latest() {
    let query = r#"SELECT "product_code" FROM "hash_testing"
        WHERE "identification_number" IN (6659253, -21, 5933116, 8257405, 3676468, 6580234, 9557717)
        AND "product_code" IN (6659253, -21, 5933116, 8257405, 3676468, 6580234, 9557717)
        AND "identification_number" < "product_code"
        AND ("product_units" <> "sys_op" OR "product_units" IS NULL)"#;

    let expected = PatternWithParams::new(
            [
                r#"SELECT "hash_testing"."product_code" FROM "hash_testing" WHERE (((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op")"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."product_code") = (?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") <> ("hash_testing"."sys_op"))"#,
                r#"or ("hash_testing"."identification_number", "hash_testing"."identification_number", "hash_testing"."product_code") = ("hash_testing"."product_code", ?, ?)"#,
                r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
                r#"and ("hash_testing"."product_units") is null)"#,
            ].join(" "),
        vec![
            Value::from(6659253_u64),
            Value::from(6659253_u64),
            Value::from(6659253_u64),
            Value::from(6659253_u64),
            Value::from(6659253_u64),
            Value::from(-21_i64),
            Value::from(6659253_u64),
            Value::from(-21_i64),
            Value::from(6659253_u64),
            Value::from(5933116_u64),
            Value::from(6659253_u64),
            Value::from(5933116_u64),
            Value::from(6659253_u64),
            Value::from(8257405_u64),
            Value::from(6659253_u64),
            Value::from(8257405_u64),
            Value::from(6659253_u64),
            Value::from(3676468_u64),
            Value::from(6659253_u64),
            Value::from(3676468_u64),
            Value::from(6659253_u64),
            Value::from(6580234_u64),
            Value::from(6659253_u64),
            Value::from(6580234_u64),
            Value::from(6659253_u64),
            Value::from(9557717_u64),
            Value::from(6659253_u64),
            Value::from(9557717_u64),
            Value::from(-21_i64),
            Value::from(6659253_u64),
            Value::from(-21_i64),
            Value::from(6659253_u64),
            Value::from(-21_i64),
            Value::from(-21_i64),
            Value::from(-21_i64),
            Value::from(-21_i64),
            Value::from(-21_i64),
            Value::from(5933116_u64),
            Value::from(-21_i64),
            Value::from(5933116_u64),
            Value::from(-21_i64),
            Value::from(8257405_u64),
            Value::from(-21_i64),
            Value::from(8257405_u64),
            Value::from(-21_i64),
            Value::from(3676468_u64),
            Value::from(-21_i64),
            Value::from(3676468_u64),
            Value::from(-21_i64),
            Value::from(6580234_u64),
            Value::from(-21_i64),
            Value::from(6580234_u64),
            Value::from(-21_i64),
            Value::from(9557717_u64),
            Value::from(-21_i64),
            Value::from(9557717_u64),
            Value::from(5933116_u64),
            Value::from(6659253_u64),
            Value::from(5933116_u64),
            Value::from(6659253_u64),
            Value::from(5933116_u64),
            Value::from(-21_i64),
            Value::from(5933116_u64),
            Value::from(-21_i64),
            Value::from(5933116_u64),
            Value::from(5933116_u64),
            Value::from(5933116_u64),
            Value::from(5933116_u64),
            Value::from(5933116_u64),
            Value::from(8257405_u64),
            Value::from(5933116_u64),
            Value::from(8257405_u64),
            Value::from(5933116_u64),
            Value::from(3676468_u64),
            Value::from(5933116_u64),
            Value::from(3676468_u64),
            Value::from(5933116_u64),
            Value::from(6580234_u64),
            Value::from(5933116_u64),
            Value::from(6580234_u64),
            Value::from(5933116_u64),
            Value::from(9557717_u64),
            Value::from(5933116_u64),
            Value::from(9557717_u64),
            Value::from(8257405_u64),
            Value::from(6659253_u64),
            Value::from(8257405_u64),
            Value::from(6659253_u64),
            Value::from(8257405_u64),
            Value::from(-21_i64),
            Value::from(8257405_u64),
            Value::from(-21_i64),
            Value::from(8257405_u64),
            Value::from(5933116_u64),
            Value::from(8257405_u64),
            Value::from(5933116_u64),
            Value::from(8257405_u64),
            Value::from(8257405_u64),
            Value::from(8257405_u64),
            Value::from(8257405_u64),
            Value::from(8257405_u64),
            Value::from(3676468_u64),
            Value::from(8257405_u64),
            Value::from(3676468_u64),
            Value::from(8257405_u64),
            Value::from(6580234_u64),
            Value::from(8257405_u64),
            Value::from(6580234_u64),
            Value::from(8257405_u64),
            Value::from(9557717_u64),
            Value::from(8257405_u64),
            Value::from(9557717_u64),
            Value::from(3676468_u64),
            Value::from(6659253_u64),
            Value::from(3676468_u64),
            Value::from(6659253_u64),
            Value::from(3676468_u64),
            Value::from(-21_i64),
            Value::from(3676468_u64),
            Value::from(-21_i64),
            Value::from(3676468_u64),
            Value::from(5933116_u64),
            Value::from(3676468_u64),
            Value::from(5933116_u64),
            Value::from(3676468_u64),
            Value::from(8257405_u64),
            Value::from(3676468_u64),
            Value::from(8257405_u64),
            Value::from(3676468_u64),
            Value::from(3676468_u64),
            Value::from(3676468_u64),
            Value::from(3676468_u64),
            Value::from(3676468_u64),
            Value::from(6580234_u64),
            Value::from(3676468_u64),
            Value::from(6580234_u64),
            Value::from(3676468_u64),
            Value::from(9557717_u64),
            Value::from(3676468_u64),
            Value::from(9557717_u64),
            Value::from(6580234_u64),
            Value::from(6659253_u64),
            Value::from(6580234_u64),
            Value::from(6659253_u64),
            Value::from(6580234_u64),
            Value::from(-21_i64),
            Value::from(6580234_u64),
            Value::from(-21_i64),
            Value::from(6580234_u64),
            Value::from(5933116_u64),
            Value::from(6580234_u64),
            Value::from(5933116_u64),
            Value::from(6580234_u64),
            Value::from(8257405_u64),
            Value::from(6580234_u64),
            Value::from(8257405_u64),
            Value::from(6580234_u64),
            Value::from(3676468_u64),
            Value::from(6580234_u64),
            Value::from(3676468_u64),
            Value::from(6580234_u64),
            Value::from(6580234_u64),
            Value::from(6580234_u64),
            Value::from(6580234_u64),
            Value::from(6580234_u64),
            Value::from(9557717_u64),
            Value::from(6580234_u64),
            Value::from(9557717_u64),
            Value::from(9557717_u64),
            Value::from(6659253_u64),
            Value::from(9557717_u64),
            Value::from(6659253_u64),
            Value::from(9557717_u64),
            Value::from(-21_i64),
            Value::from(9557717_u64),
            Value::from(-21_i64),
            Value::from(9557717_u64),
            Value::from(5933116_u64),
            Value::from(9557717_u64),
            Value::from(5933116_u64),
            Value::from(9557717_u64),
            Value::from(8257405_u64),
            Value::from(9557717_u64),
            Value::from(8257405_u64),
            Value::from(9557717_u64),
            Value::from(3676468_u64),
            Value::from(9557717_u64),
            Value::from(3676468_u64),
            Value::from(9557717_u64),
            Value::from(6580234_u64),
            Value::from(9557717_u64),
            Value::from(6580234_u64),
            Value::from(9557717_u64),
            Value::from(9557717_u64),
            Value::from(9557717_u64),
            Value::from(9557717_u64),
        ],
    );
    check_sql_with_snapshot(query, expected, Snapshot::Latest);
}

#[test]
fn selection2_oldest() {
    let query = r#"SELECT "product_code" FROM "hash_testing"
        WHERE "identification_number" IN (6659253, -21, 5933116, 8257405, 3676468, 6580234, 9557717)
        AND "product_code" IN (6659253, -21, 5933116, 8257405, 3676468, 6580234, 9557717)
        AND "identification_number" < "product_code"
        AND ("product_units" <> "sys_op" OR "product_units" IS NULL)"#;

    let expected = PatternWithParams::new(
        format!(
            "{} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {}",
            r#"SELECT "hash_testing"."product_code" FROM "hash_testing""#,
            r#"WHERE ((((((("hash_testing"."identification_number") = (?)"#,
            r#"or ("hash_testing"."identification_number") = (?))"#,
            r#"or ("hash_testing"."identification_number") = (?))"#,
            r#"or ("hash_testing"."identification_number") = (?))"#,
            r#"or ("hash_testing"."identification_number") = (?))"#,
            r#"or ("hash_testing"."identification_number") = (?))"#,
            r#"or ("hash_testing"."identification_number") = (?))"#,
            r#"and ((((((("hash_testing"."product_code") = (?)"#,
            r#"or ("hash_testing"."product_code") = (?))"#,
            r#"or ("hash_testing"."product_code") = (?))"#,
            r#"or ("hash_testing"."product_code") = (?))"#,
            r#"or ("hash_testing"."product_code") = (?))"#,
            r#"or ("hash_testing"."product_code") = (?))"#,
            r#"or ("hash_testing"."product_code") = (?))"#,
            r#"and ("hash_testing"."identification_number") < ("hash_testing"."product_code")"#,
            r#"and (("hash_testing"."product_units") <> ("hash_testing"."sys_op")"#,
            r#"or ("hash_testing"."product_units") is null)"#,
        ),
        vec![
            Value::from(6659253_u64),
            Value::from(-21_i64),
            Value::from(5933116_u64),
            Value::from(8257405_u64),
            Value::from(3676468_u64),
            Value::from(6580234_u64),
            Value::from(9557717_u64),
            Value::from(6659253_u64),
            Value::from(-21_i64),
            Value::from(5933116_u64),
            Value::from(8257405_u64),
            Value::from(3676468_u64),
            Value::from(6580234_u64),
            Value::from(9557717_u64),
        ],
    );
    check_sql_with_snapshot(query, expected, Snapshot::Oldest);
}
